
options
{
}

// -----------------------------------------------------------------------------

{
	import antlr.CommonToken;

	import java.util.List;
	import java.util.Vector;
	import java.util.Map;
	import java.util.LinkedHashMap;
	import java.io.InputStream;
	import java.io.File;
	import java.io.FileInputStream;
	import java.io.FileNotFoundException;

	@SuppressWarnings( "all" )
}

class TraceParser extends Parser;

options
{
    k = 2;
    buildAST = false;
    defaultErrorHandler = false;
}

tokens
{
	SYSTEM_CALL_ENTRY;
	SYSTEM_CALL_TRACE;
}

strace : ( entry )* EOF	;

entry
   :  ( GT | LESSTHAN )?
      sequence_number
      ( STAR )?
      process_id
      thread_id
      syscall_name
      LBRACKET argument_count RBRACKET
      LPAREN ( argument_list )? RPAREN
      ( return_value )?
   ;

sequence_number
	: numeric_value
	;

process_id
	: numeric_value
	;

thread_id
	: numeric_value
	;

syscall_name
	: IDENTIFIER
	;

argument_count
	: numeric_value
	;

argument_list
	: argument ( COMMA argument )*
	;

argument
	: simple_argument
	| LESSTHAN simple_argument BOR simple_argument GT
	;

simple_argument
	: UNKNOWN_VALUE
	| numeric_value
	| STRING
	| IDENTIFIER
	| IDENTIFIER PLUS HEX_NUMBER
	| LPAREN argument_list RPAREN
	| LCURLY ( argument_list | named_argument_list | meta_argument_list ) RCURLY
	;

protected
named_argument_list
	: named_argument ( COMMA named_argument )*
	;

protected
named_argument
	: IDENTIFIER ASSIGN argument
	;

protected
meta_argument_list
	: IDENTIFIER COLON argument
		COMMA IDENTIFIER COLON argument
		COMMA IDENTIFIER COLON argument
	;

protected
return_value
	: EQUAL ( HEX_NUMBER | IDENTIFIER )
	;

protected
numeric_value
	: NUMBER
	| HEX_NUMBER
	;

// -----------------------------------------------------------------------------

{
	@SuppressWarnings({"all"})
}

class TraceLexer extends Lexer;

options
{
    k = 4;
    charVocabulary = '\0' .. '\377';
}

// tokens
// {
// }

// Operators
LPAREN			:	'('		;
RPAREN			:	')'		;
LBRACKET		:	'['		;
RBRACKET		:	']'		;
LCURLY			:	'{'		;
RCURLY			:	'}'		;
COLON			:	':'		;
COMMA			:	','		;
DOT 			:	'.'		;
ASSIGN			:	'='		;
EQUAL			:	"=="	;
LNOT			:	'!'		;
BNOT			:	'~'		;
NOT_EQUAL		:	"!="	;
DIV	    		:	'/'		;
DIV_ASSIGN		:	"/="	;
PLUS			:	'+'		;
PLUS_ASSIGN		:	"+="	;
INC		    	:	"++"	;
// MINUS			:	'-'		;
MINUS_ASSIGN	:	"-="	;
DEC			    :	"--"	;
STAR			:	'*'		;
STAR_ASSIGN		:	"*="	;
MOD 			:	'%'		;
MOD_ASSIGN		:	"%="	;
SR	    		:	">>"	;
SR_ASSIGN		:	">>="	;
BSR		    	:	">>>"	;
BSR_ASSIGN		:	">>>="	;
GE			    :	">="	;
GT  			:	">"		;
SL	    		:	"<<"	;
SL_ASSIGN		:	"<<="	;
LE		    	:	"<="	;
LESSTHAN		:	'<'		;
BXOR			:	'^'		;
BXOR_ASSIGN		:	"^="	;
BOR			    :	'|'		;
BOR_ASSIGN		:	"|="	;
LOR 			:	"||"	;
BAND			:	'&'		;
BAND_ASSIGN		:	"&="	;
LAND			:	"&&"	;
SEMI			:	';' 	;
QUOTE      		:	'\''    ;
DOUBLEQUOTE    	:   '"'     ;
// BACKSLASH      	:   '\\'    ;
DOLLARSIGN     	:   '$'     ;
AT             	:   '@'     ;
RIGHTARROW		:	"->"	;
// QUESTION        :   '?'		;
DEFINE          :	":="	;

// Whitespace -- ignored
WS
    :	( ' '
        | '\t'
        | '\f'
        |   ( options { generateAmbigWarnings = false; }
            : "\r\n" // DOS
            | '\n'
            | '\r'
            )
            { newline(); }
        )+
        { $setType( antlr.Token.SKIP ); }
    ;

NUMBER
    :   ( '-' )? '1' .. '9' ( '0' .. '9' )*
    |	'0'
    ;

HEX_NUMBER
    :   "0x" ( '0' .. '9' | 'A' .. 'Z' | 'a' .. 'z' )+
    ;

STRING
    :	'"' ( ~( '"' | '\\' ) | ( '\\' . ) )* '"'
    ;

IDENTIFIER
    :   ( 'a' .. 'z' | 'A' .. 'Z' )
        ( 'a' .. 'z' | 'A' .. 'Z' | '0' .. '9' | '_' )*
    ;

UNKNOWN_VALUE
    :   "???"
    ;

// -----------------------------------------------------------------------------
